import os
import sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

import time

class Stats:
    __slots__ = (
        "volume",
        "count",
        "profit_count",
        "loss_count",
        "win_rate",
        "profit_value",
        "loss_value",
        "net_pnl",  # 价差净收益
        "base_funding_fee",
        "hedge_funding_fee",
        "funding_pnl",
        "leg_count",
        "leg_volume",
        "leg_pnl",
        "total_pnl",  # net_pnl + funding_pnl + leg_pnl
        "fee",
        "rebate",
    )

    def __init__(self):
        self.volume = 0.0
        self.count = 0
        self.profit_count = 0
        self.loss_count = 0
        self.win_rate = 0.0
        self.profit_value = 0.0
        self.loss_value = 0.0
        self.net_pnl = 0.0
        self.base_funding_fee = 0.0
        self.hedge_funding_fee = 0.0
        self.funding_pnl = 0.0
        self.leg_count = 0
        self.leg_volume = 0.0
        self.leg_pnl = 0.0
        self.total_pnl = 0.0
        self.fee = 0.0
        self.rebate = 0.0


class Statistics:
    def __init__(self):
        self.now_balance = 10000
        self.max_balance = 10000
        self.today_start_balance = 10000

        self.start_balance = 10000
        self.start_time = int(time.time() * 1000)

        self.fee_rate = 0.0006

        self.symbol_stats = {symbol: Stats() for symbol in ['BTC_USDT']}

        self.total_stats = Stats()



import os
import sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from typing import List, Dict, Any
from utils.position_manager import HedgePositionManager
from utils.loader import get_config_loader

# 获取配置加载器
_config = get_config_loader()

def symbol_exposure_check(symbols: List[str], position_manager: HedgePositionManager) -> bool:
    """
    检查 symbol 的敞口是否正常
    如果检查 symbol 的仓位发生了较大敞口，则应该将 symbol 加到 result 中返回
    """
    result = {'exposure': list()}
    if not symbols or len(symbols) <= 0:
        return result
    
    for symbol in symbols:
        pos_0, pos_1 = position_manager.get_hedge_positions(symbol)
        pos_0_size = pos_0.amount
        pos_1_size = pos_1.amount

        pos_size_delta = abs(pos_0_size - pos_1_size)
        threshold = _config.get_global_config()['abnormal_position_exposure_threshold']

        if pos_size_delta / pos_1_size > threshold:
            result['exposure'].append(symbol)
        
    return result

def symbol_notional_exposure_check(symbols: List[str], bbo: List[Dict[str, Any]], position_manager: HedgePositionManager) -> bool:
    """
    检查 symbol 的名义市值敞口是否正常
    如果检查 symbol 的仓位发生了较大敞口，则应该将 symbol 加到 result 中返回
    """
    result = {'exposure': list()}
    if not symbols or not position_manager:
        return result
    
    for symbol in symbols:
        pos_0, pos_1 = position_manager.get_hedge_positions(symbol)
        taker_notional = pos_0.amount * (bbo[0][symbol]['ask_price'] + bbo[0][symbol]['bid_price']) * 0.5
        maker_notional = pos_1.amount * (bbo[1][symbol]['ask_price'] + bbo[1][symbol]['bid_price']) * 0.5
        threshold = _config.get_global_config()['abnormal_position_exposure_threshold']

        if taker_notional / maker_notional - 1 > threshold or maker_notional / taker_notional - 1 > threshold:
            result['exposure'].append(symbol)
    
    return result
    

def symbol_single_leg_check(symbols: List[str], position_manager: HedgePositionManager) -> bool:
    """
    检查 symbol 的单腿是否正常
    如果检查 symbol 发生了单腿，则应该将该 symbol 加到 result 中返回
    """
    result = {'single_leg': list()}
    if not symbols or len(symbols) <= 0:
        return result

    for symbol in symbols:
        pos_0, pos_1 = position_manager.get_hedge_positions(symbol)
        pos_0_size = pos_0.amount
        pos_1_size = pos_1.amount

        if pos_0_size == 0 or pos_1_size == 0:
            result['single_leg'].append(symbol)
    
    return result


from typing import List, Dict, Any
import time
from utils.loader import get_config_loader

# 获取配置加载器
_config = get_config_loader()

def bbo_latency_check(bbo: List[Dict[str, Any]]) -> bool:
    """
    检查相同 symbol 的行情延迟是否异常
    """
    if not bbo or len(bbo) != 2 or not bbo[0] or not bbo[1]:
        return False
    
    taker_bbo = bbo[0]
    maker_bbo = bbo[1]
    
    taker_timestamp = taker_bbo.get('timestamp', 0)
    maker_timestamp = maker_bbo.get('timestamp', 0)

    current_timestamp = int(time.time() * 1000)
    threshold = _config.get_tick_config()['abnormal_bbo_latency_threshold']

    if abs(taker_timestamp - current_timestamp) > threshold or abs(maker_timestamp - current_timestamp) > threshold:
        return False

    return True





from typing import Any, Dict, Optional, Tuple
import time
from utils.loader import get_config_loader

# 获取配置加载器
_config = get_config_loader()

# 代码用于计算交易所的订单异常情况，例如订单连续失败，订单连续产生较大下单延迟等。

def exchange_order_failure_check(index: int, order_receipt: Dict[str, Any]) -> bool:
    """
    检查交易所订单回执是否失败
    """
    if not isinstance(order_receipt, dict):
        return False
    return "Ok" in order_receipt
    
def exchange_order_slippage_check(order: Dict[str, Any], threshold_bps: Optional[float] = None) -> Tuple[bool, Optional[float], float]:
    """
    交易所层面滑点检查
    - 以下单 price 与成交均价 filled_avg_price 计算滑点（bps）
    - 如果未指定 threshold_bps，则从配置文件读取
    """
    if threshold_bps is None:
        threshold_bps = _config.get_order_config()['abnormal_order_slippage_threshold']
    try:
        price = float(order.get("price"))
        filled = float(order.get("filled", 0.0) or 0.0)
        filled_avg_price = float(order.get("filled_avg_price")) if filled > 0 else None
    except Exception:
        return False, None, threshold_bps

    if not price or filled_avg_price is None or price <= 0:
        return False, None, threshold_bps
    slippage_bps = abs(filled_avg_price - price) / price * 10000.0
    return slippage_bps > threshold_bps, slippage_bps, threshold_bps
    
def exchange_order_latency_check(order: Dict[str, Any], threshold_ms: Optional[int] = None) -> Tuple[bool, Optional[int], int]:
    """
    交易所层面下单延迟检查
    - 以当前时间与 order['timestamp'] 相差计算下单延迟
    - 如果未指定 threshold_ms，则从配置文件读取
    """
    if threshold_ms is None:
        threshold_ms = _config.get_order_config()['abnormal_order_latency_threshold']
    try:
        ts = int(order.get("timestamp"))
    except Exception:
        return False, None, threshold_ms
    now_ms = int(time.time() * 1000)
    latency = max(0, now_ms - ts)
    return latency > threshold_ms, latency, threshold_ms




import os
import sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from typing import Tuple

class Position:
    def __init__(self):
        self.amount = 10
        self.avg_price = 0
        self.side = ''
        self.hedge_index = 0
        self.update_time = 0

    def __repr__(self):
        return f"Position(amount={self.amount}, avg_price={self.avg_price}, side={self.side}, hedge_index={self.hedge_index})"

class HedgePositionManager:
    def __init__(self):
        print(f"Mock HedgePositionManager class 初始化")

        self.symbols = ''
        self.base_index = 2
        self.positions = [{symbol: Position() for symbol in ['BTC_USDT']} for _ in range(self.base_index)]

    def get_hedge_positions(self, symbol: str) -> Tuple[Position, Position]:
        return Position(), Position()




import os
import sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from typing import List, Dict, Any
from utils.position_manager import HedgePositionManager
from utils.loader import get_config_loader

# 获取配置加载器
_config = get_config_loader()

def leverage_check(symbols: List[str], bbo: List[Dict[str, Any]], balances: List[Dict[str, Any]], position_manager: HedgePositionManager) -> bool:
    """
    同时检查 symbol 和 账户杠杆是否正常
    """
    result = {'account_leverage': False, 'symbol_leverage': {}}
    if not symbols or not balances or len(balances) < 2:
        return result
    
    # 检查数据完整性
    if not balances[0] or 'balance' not in balances[0]:
        return result
    if not balances[1] or 'balance' not in balances[1]:
        return result
    
    taker_balance = balances[0]['balance']
    maker_balance = balances[1]['balance']
    
    # 检查余额是否为0，避免除零错误
    if taker_balance == 0 and maker_balance == 0:
        return result
    
    total_notional = 0

    for symbol in symbols:
        taker_bbo = bbo[0][symbol]
        maker_bbo = bbo[1][symbol]

        taker_mid_price = (taker_bbo['bid_price'] + taker_bbo['ask_price']) / 2
        maker_mid_price = (maker_bbo['bid_price'] + maker_bbo['ask_price']) / 2
        
        pos_0, pos_1 = position_manager.get_hedge_positions(symbol)
        pos_0_size = pos_0.amount
        pos_1_size = pos_1.amount

        if pos_0_size == 0 or pos_1_size == 0:
            continue

        taker_notional = pos_0_size * taker_mid_price
        maker_notional = pos_1_size * maker_mid_price
        total_notional += taker_notional + maker_notional

        global_config = _config.get_global_config()
        max_symbol_leverage = global_config['max_symbol_leverage']
        max_account_leverage = global_config['max_account_leverage']

        if (taker_notional + maker_notional) / (taker_balance + maker_balance) >= max_symbol_leverage:
            # 当 symbol 的持仓名义市值超过阈值，则添加到风控结果。
            result['symbol_leverage'][symbol] = True
        
        if total_notional / (taker_balance + maker_balance) >= max_account_leverage:
            result['account_leverage'] = True

    return result




from typing import List, Dict, Any
from utils.loader import get_config_loader

# 获取配置加载器
_config = get_config_loader()

def position_liquidation_price_check(positions: List[List[Dict[str, Any]]], bbo: List[Dict[str, Any]]) -> bool:
    """
    检查所有 position 当前 symbol 的价格是否接近强制平仓价格。
    position 是 [[{}] [{}]] 来存储全部交易所的 全部 position
    需要先遍历交易所，再遍历 position，再获取 bbo 数据
    """
    result = {'Info': dict()}

    exchange_count = len(positions)

    threshold = _config.get_global_config()['liquidation_price_warning_threshold']
    for exchange in range(exchange_count):
        for position in positions[exchange]:
            symbol = position['symbol']
            mid_price = (bbo[exchange][symbol]['bid_price'] + bbo[exchange][symbol]['ask_price']) / 2
            if abs(position['liquidation_price'] - mid_price) / position['liquidation_price'] > threshold:
                result['Info'][symbol] = {'liquidation_price': position['liquidation_price'], 'mid_price': mid_price}

    return result